@@ -2,7 +2,7 @@
 /*
  * GData Client
  * Copyright (C) Thibault Saunier 2009 <saunierthibault@gmail.com>
- * Copyright (C) Red Hat, Inc. 2015
+ * Copyright (C) Red Hat, Inc. 2015, 2016
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -570,6 +570,7 @@ parse_json (GDataParsable *parsable, JsonReader *reader, gpointer user_data, GEr
 	gboolean success = TRUE;
 	gchar *alternate_uri = NULL;
 	gchar *kind = NULL;
+	gchar *quota_used = NULL;
 	gint64 published;
 	gint64 updated;
 
@@ -607,6 +608,18 @@ parse_json (GDataParsable *parsable, JsonReader *reader, gpointer user_data, GEr
 		if (success)
 			_gdata_entry_set_updated (GDATA_ENTRY (parsable), updated);
 		return success;
+	} else if (gdata_parser_string_from_json_member (reader, "quotaBytesUsed", P_DEFAULT, &quota_used, &success, error) == TRUE) {
+		gchar *end_ptr;
+		guint64 val;
+
+		/* Even though ‘quotaBytesUsed’ is documented as long,
+		 * it is actually a string in the JSON.
+		 */
+		val = g_ascii_strtoull (quota_used, &end_ptr, 10);
+		if (*end_ptr == '\0')
+			priv->quota_used = (goffset) val;
+		g_free (quota_used);
+		return success;
 	} else if (gdata_parser_boolean_from_json_member (reader, "shared", P_DEFAULT, &shared, &success, error) == TRUE) {
 		if (success && shared) {
 			category = gdata_category_new ("http://schemas.google.com/g/2005/labels#shared", GDATA_CATEGORY_SCHEMA_LABELS, "shared");
@@ -803,30 +816,21 @@ get_content_type (void)
 static void
 get_json (GDataParsable *parsable, JsonBuilder *builder)
 {
-	GList *categories;
 	GList *i;
 	GList *parent_folders_list;
+	const gchar *mime_type;
 
 	GDATA_PARSABLE_CLASS (gdata_documents_entry_parent_class)->get_json (parsable, builder);
 
 	/* Inserting files: https://developers.google.com/drive/v2/reference/files/insert */
 
-	categories = gdata_entry_get_categories (GDATA_ENTRY (parsable));
-	for (i = categories; i != NULL; i = i->next) {
-		GDataCategory *category = GDATA_CATEGORY (i->data);
-		const gchar *label;
-		const gchar *scheme;
-
-		label = gdata_category_get_label (category);
-		scheme = gdata_category_get_scheme (category);
-		if (label != NULL && label[0] != '\0' && g_strcmp0 (scheme, "http://schemas.google.com/g/2005#kind") == 0) {
-			json_builder_set_member_name (builder, "mimeType");
-			json_builder_add_string_value (builder, label);
-			break;
-		}
+	mime_type = gdata_documents_utils_get_content_type (GDATA_DOCUMENTS_ENTRY (parsable));
+	if (mime_type != NULL) {
+		json_builder_set_member_name (builder, "mimeType");
+		json_builder_add_string_value (builder, mime_type);
 	}
 
-	/* Upload to a folder: https://developers.google.com/drive/web/folder */
+	/* Upload to a folder: https://developers.google.com/drive/v2/web/folder */
 
 	json_builder_set_member_name (builder, "parents");
 	json_builder_begin_array (builder);
